@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,11 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
-
-#if HAVE_GDK_IMLIB
-#  include <gdk_imlib.h>
-#endif
+#include <gtk/gtkstock.h>
+#include <gtk/gtkaccelmap.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,6 +39,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <signal.h>
+#include <dirent.h>
 
 #if HAVE_LOCALE_H
 #  include <locale.h>
@@ -49,7 +49,6 @@
 #  include <gpgme.h>
 #endif
 
-#include "intl.h"
 #include "main.h"
 #include "mainwindow.h"
 #include "folderview.h"
@@ -67,12 +66,14 @@
 #include "alertpanel.h"
 #include "statusbar.h"
 #include "addressbook.h"
+#include "addrindex.h"
 #include "compose.h"
 #include "folder.h"
 #include "setup.h"
 #include "utils.h"
 #include "gtkutils.h"
 #include "socket.h"
+#include "stock_pixmap.h"
 
 #if USE_GPGME
 #  include "rfc2015.h"
@@ -118,17 +119,13 @@ static void lock_socket_input_cb	(gpointer	   data,
 					 GdkInputCondition condition);
 static gchar *get_socket_name		(void);
 
+static void migrate_old_config		(void);
+
 static void open_compose_new		(const gchar	*address,
 					 GPtrArray	*attach_files);
 
 static void send_queue			(void);
 
-#if 0
-/* for gettext */
-_("File `%s' already exists.\n"
-  "Can't create folder.")
-#endif
-
 #define MAKE_DIR_IF_NOT_EXIST(dir) \
 { \
 	if (!is_dir_exist(dir)) { \
@@ -149,9 +146,11 @@ int main(int argc, char *argv[])
 	gchar *userrc;
 	MainWindow *mainwin;
 	FolderView *folderview;
+	GdkPixbuf *icon;
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset(PACKAGE, CS_UTF_8);
 	textdomain(PACKAGE);
 
 	prog_version = PROG_VERSION;
@@ -182,12 +181,6 @@ int main(int argc, char *argv[])
 		g_error(_("g_thread is not supported by glib.\n"));
 #endif
 
-#if HAVE_GDK_IMLIB
-	gdk_imlib_init();
-	gtk_widget_push_visual(gdk_imlib_get_visual());
-	gtk_widget_push_colormap(gdk_imlib_get_colormap());
-#endif
-
 #if USE_SSL
 	ssl_init();
 #endif
@@ -195,12 +188,12 @@ int main(int argc, char *argv[])
 	srandom((gint)time(NULL));
 
 	/* parse gtkrc files */
-	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc",
+	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc-2.0",
 			     NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
 	userrc = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, ".gtk",
-			     G_DIR_SEPARATOR_S, "gtkrc", NULL);
+			     G_DIR_SEPARATOR_S, "gtkrc-2.0", NULL);
 	gtk_rc_parse(userrc);
 	g_free(userrc);
 	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "gtkrc", NULL);
@@ -208,7 +201,7 @@ int main(int argc, char *argv[])
 	g_free(userrc);
 
 	userrc = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, MENU_RC, NULL);
-	gtk_item_factory_parse_rc(userrc);
+	gtk_accel_map_load(userrc);
 	g_free(userrc);
 
 	CHDIR_RETURN_VAL_IF_FAIL(get_home_dir(), 1);
@@ -218,12 +211,47 @@ int main(int argc, char *argv[])
 		if (rename(RC_DIR, RC_DIR ".bak") < 0)
 			FILE_OP_ERROR(RC_DIR, "rename");
 	}
-	MAKE_DIR_IF_NOT_EXIST(RC_DIR);
+
+	/* migration from ~/.sylpheed to ~/.sylpheed-2.0 */
+	if (!is_dir_exist(RC_DIR)) {
+		const gchar *envstr;
+		AlertValue val;
+
+		/* check for filename encoding */
+		if (conv_get_locale_charset() != C_UTF_8) {
+			envstr = g_getenv("G_FILENAME_ENCODING");
+			if (!envstr)
+				envstr = g_getenv("G_BROKEN_FILENAMES");
+			if (!envstr) {
+				val = alertpanel(_("Filename encoding"),
+						 _("The locale encoding is not UTF-8, but the environmental variable G_FILENAME_ENCODING is not set.\n"
+						   "If the locale encoding is used for file name or directory name, it will not work correctly.\n"
+						   "In that case, you must set the following environmental variable (see README for detail):\n"
+						   "\n"
+						   "\tG_FILENAME_ENCODING=@locale\n"
+						   "\n"
+						   "Continue?"),
+						 GTK_STOCK_OK, GTK_STOCK_QUIT,
+						 NULL);
+				if (G_ALERTDEFAULT != val)
+					return 1;
+			}
+		}
+		if (make_dir(RC_DIR) < 0)
+			return 1;
+		if (is_dir_exist(OLD_RC_DIR))
+			migrate_old_config();
+	}
+
 	MAKE_DIR_IF_NOT_EXIST(get_imap_cache_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_news_cache_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_mime_tmp_dir());
 	MAKE_DIR_IF_NOT_EXIST(get_tmp_dir());
-	MAKE_DIR_IF_NOT_EXIST(RC_DIR G_DIR_SEPARATOR_S "uidl");
+	MAKE_DIR_IF_NOT_EXIST(RC_DIR G_DIR_SEPARATOR_S UIDL_DIR);
+
+	/* remove temporary files */
+	remove_all_files(get_tmp_dir());
+	remove_all_files(get_mime_tmp_dir());
 
 	if (is_file_exist(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log")) {
 		if (rename(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log",
@@ -232,25 +260,32 @@ int main(int argc, char *argv[])
 	}
 	set_log_file(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log");
 
+	set_ui_update_func(gtkut_events_flush);
+	set_progress_func(main_window_progress_show);
+
 	prefs_common_read_config();
 
 #if USE_GPGME
-	if (gpgme_check_version("0.4.5")) {  /* Also does some gpgme init */
-        gpgme_engine_info_t engineInfo;
+	if (gpgme_check_version("0.4.5")) {
+		/* Also does some gpgme init */
+	        gpgme_engine_info_t engineInfo;
+
 		rfc2015_disable_all();
 
-        gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
-        gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
+		gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
+		gpgme_set_locale(NULL, LC_MESSAGES,
+				 setlocale(LC_MESSAGES, NULL));
 
-        if (!gpgme_get_engine_info(&engineInfo)) {
-            while (engineInfo) {
-                debug_print("GpgME Protocol: %s\n      Version: %s\n",
-                    gpgme_get_protocol_name(engineInfo->protocol),
-                    engineInfo->version);
-                engineInfo = engineInfo->next;
-            }
-        }
-    } else {
+		if (!gpgme_get_engine_info(&engineInfo)) {
+			while (engineInfo) {
+				debug_print("GpgME Protocol: %s\n      Version: %s\n",
+					    gpgme_get_protocol_name
+						(engineInfo->protocol),
+					    engineInfo->version);
+				engineInfo = engineInfo->next;
+			}
+		}
+	} else {
 		if (prefs_common.gpg_warning) {
 			AlertValue val;
 
@@ -263,12 +298,12 @@ int main(int argc, char *argv[])
 				prefs_common.gpg_warning = FALSE;
 		}
 	}
-    /* FIXME: This function went away.  We can either block until gpgme
-     * operations finish (currently implemented) or register callbacks
-     * with the gtk main loop via the gpgme io callback interface instead.
-     *
+	/* FIXME: This function went away.  We can either block until gpgme
+	 * operations finish (currently implemented) or register callbacks
+	 * with the gtk main loop via the gpgme io callback interface instead.
+	 *
 	 * gpgme_register_idle(idle_function_for_gpgme);
-     */
+	 */
 #endif
 
 	sock_set_io_timeout(prefs_common.io_timeout_secs);
@@ -278,6 +313,8 @@ int main(int argc, char *argv[])
 	prefs_display_header_read_config();
 
 	gtkut_widget_init();
+	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED, &icon);
+	gtk_window_set_default_icon(icon);
 
 	mainwin = main_window_create
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
@@ -318,7 +355,7 @@ int main(int argc, char *argv[])
 	else if (cmd.receive)
 		inc_mail(mainwin);
 	else
-		gtk_widget_grab_focus(folderview->ctree);
+		gtk_widget_grab_focus(folderview->treeview);
 
 	if (cmd.compose)
 		open_compose_new(cmd.compose_mailto, cmd.attach_files);
@@ -417,19 +454,19 @@ static void parse_cmd_opt(int argc, char *argv[])
 			g_print(_("Usage: %s [OPTION]...\n"),
 				g_basename(argv[0]));
 
-			puts(_("  --compose [address]    open composition window"));
-			puts(_("  --attach file1 [file2]...\n"
-			       "                         open composition window with specified files\n"
-			       "                         attached"));
-			puts(_("  --receive              receive new messages"));
-			puts(_("  --receive-all          receive new messages of all accounts"));
-			puts(_("  --send                 send all queued messages"));
-			puts(_("  --status [folder]...   show the total number of messages"));
-			puts(_("  --status-full [folder]...\n"
-			       "                         show the status of each folder"));
-			puts(_("  --debug                debug mode"));
-			puts(_("  --help                 display this help and exit"));
-			puts(_("  --version              output version information and exit"));
+			g_print("%s\n", _("  --compose [address]    open composition window"));
+			g_print("%s\n", _("  --attach file1 [file2]...\n"
+				"                         open composition window with specified files\n"
+				"                         attached"));
+			g_print("%s\n", _("  --receive              receive new messages"));
+			g_print("%s\n", _("  --receive-all          receive new messages of all accounts"));
+			g_print("%s\n", _("  --send                 send all queued messages"));
+			g_print("%s\n", _("  --status [folder]...   show the total number of messages"));
+			g_print("%s\n", _("  --status-full [folder]...\n"
+				"                         show the status of each folder"));
+			g_print("%s\n", _("  --debug                debug mode"));
+			g_print("%s\n", _("  --help                 display this help and exit"));
+			g_print("%s\n", _("  --version              output version information and exit"));
 
 			exit(1);
 		}
@@ -460,7 +497,8 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	if (compose_get_compose_list()) {
 		if (alertpanel(_("Notice"),
 			       _("Composing message exists. Really quit?"),
-			       _("OK"), _("Cancel"), NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
+		    != G_ALERTDEFAULT)
 			return;
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
@@ -468,7 +506,8 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	if (prefs_common.warn_queued_on_exit && get_queued_message_num() > 0) {
 		if (alertpanel(_("Queued messages"),
 			       _("Some unsent messages are queued. Exit now?"),
-			       _("OK"), _("Cancel"), NULL) != G_ALERTDEFAULT)
+			       GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
+		    != G_ALERTDEFAULT)
 			return;
 		manage_window_focus_in(mainwin->window, NULL, NULL);
 	}
@@ -490,10 +529,11 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	addressbook_export_to_file();
 
 	filename = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, MENU_RC, NULL);
-	gtk_item_factory_dump_rc(filename, NULL, TRUE);
+	gtk_accel_map_save(filename);
 	g_free(filename);
 
-	/* delete temporary files */
+	/* remove temporary files */
+	remove_all_files(get_tmp_dir());
 	remove_all_files(get_mime_tmp_dir());
 
 	close_log_file();
@@ -703,6 +743,85 @@ static void lock_socket_input_cb(gpointer data,
 	fd_close(sock);
 }
 
+static void migrate_old_config(void)
+{
+	DIR *dp;
+	struct dirent *d;
+	GPatternSpec *pspec;
+
+	if (alertpanel(_("Migration of configuration"),
+		       _("The previous version of configuration found.\n"
+			 "Do you want to migrate it?"),
+		       GTK_STOCK_YES, GTK_STOCK_NO, NULL) != G_ALERTDEFAULT)
+		return;
+
+	debug_print("Migrating old configuration...\n");
+
+#define COPY_FILE(rcfile)						\
+	if (is_file_exist(OLD_RC_DIR G_DIR_SEPARATOR_S rcfile)) {	\
+		conv_copy_file(OLD_RC_DIR G_DIR_SEPARATOR_S rcfile,	\
+			       RC_DIR G_DIR_SEPARATOR_S rcfile,		\
+			       conv_get_locale_charset_str());		\
+	}
+
+	COPY_FILE(ACCOUNT_RC);
+	COPY_FILE(ACTIONS_RC);
+	COPY_FILE(COMMON_RC);
+	COPY_FILE(CUSTOM_HEADER_RC);
+	COPY_FILE(DISPLAY_HEADER_RC);
+	COPY_FILE(FILTER_HEADER_RC);
+	COPY_FILE(COMMAND_HISTORY);
+
+#undef COPY_FILE
+
+	if (is_file_exist(OLD_RC_DIR G_DIR_SEPARATOR_S FILTER_LIST))
+		copy_file(OLD_RC_DIR G_DIR_SEPARATOR_S FILTER_LIST,
+			  RC_DIR G_DIR_SEPARATOR_S FILTER_LIST, FALSE);
+	if (is_file_exist(OLD_RC_DIR G_DIR_SEPARATOR_S FOLDER_LIST))
+		copy_file(OLD_RC_DIR G_DIR_SEPARATOR_S FOLDER_LIST,
+			  RC_DIR G_DIR_SEPARATOR_S FOLDER_LIST, FALSE);
+	if (is_file_exist(OLD_RC_DIR G_DIR_SEPARATOR_S "mime.types"))
+		copy_file(OLD_RC_DIR G_DIR_SEPARATOR_S "mime.types",
+			  RC_DIR G_DIR_SEPARATOR_S "mime.types", FALSE);
+
+	if (is_dir_exist(OLD_RC_DIR G_DIR_SEPARATOR_S TEMPLATE_DIR))
+		conv_copy_dir(OLD_RC_DIR G_DIR_SEPARATOR_S TEMPLATE_DIR,
+			      RC_DIR G_DIR_SEPARATOR_S TEMPLATE_DIR,
+			      conv_get_locale_charset_str());
+	if (is_dir_exist(OLD_RC_DIR G_DIR_SEPARATOR_S UIDL_DIR))
+		copy_dir(OLD_RC_DIR G_DIR_SEPARATOR_S UIDL_DIR,
+			 RC_DIR G_DIR_SEPARATOR_S UIDL_DIR);
+
+	if (!is_file_exist(OLD_RC_DIR G_DIR_SEPARATOR_S ADDRESSBOOK_INDEX_FILE))
+		return;
+
+	if ((dp = opendir(OLD_RC_DIR)) == NULL) {
+		FILE_OP_ERROR(OLD_RC_DIR, "opendir");
+		return;
+	}
+
+	pspec = g_pattern_spec_new("addrbook-*.xml");
+
+	while ((d = readdir(dp)) != NULL) {
+		if (g_pattern_match_string(pspec, d->d_name)) {
+			gchar *old_file;
+			gchar *new_file;
+
+			old_file = g_strconcat(OLD_RC_DIR G_DIR_SEPARATOR_S,
+					       d->d_name, NULL);
+			new_file = g_strconcat(RC_DIR G_DIR_SEPARATOR_S,
+					       d->d_name, NULL);
+			copy_file(old_file, new_file, FALSE);
+			g_free(new_file);
+			g_free(old_file);
+		}
+	}
+
+	g_pattern_spec_free(pspec);
+
+	closedir(dp);
+}
+
 static void open_compose_new(const gchar *address, GPtrArray *attach_files)
 {
 	gchar *addr = NULL;
@@ -729,7 +848,8 @@ static void send_queue(void)
 			gint ret;
 
 			ret = procmsg_send_queue(folder->queue,
-						 prefs_common.savemsg);
+						 prefs_common.savemsg,
+						 prefs_common.filter_sent);
 			statusbar_pop_all();
 			if (ret > 0)
 				folder_item_scan(folder->queue);
