@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2004 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkwindow.h>
@@ -36,12 +37,14 @@
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkbutton.h>
-#include <gtk/gtkctree.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtktreemodel.h>
+#include <gtk/gtktreeselection.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "intl.h"
 #include "main.h"
 #include "summary_search.h"
 #include "summaryview.h"
@@ -53,64 +56,103 @@
 #include "manage_window.h"
 #include "alertpanel.h"
 
-static GtkWidget *window;
-static GtkWidget *bool_optmenu;
-static GtkWidget *from_entry;
-static GtkWidget *to_entry;
-static GtkWidget *subject_entry;
-static GtkWidget *body_entry;
-static GtkWidget *case_checkbtn;
-static GtkWidget *backward_checkbtn;
-static GtkWidget *all_checkbtn;
-static GtkWidget *search_btn;
-static GtkWidget *clear_btn;
-static GtkWidget *close_btn;
+static struct SummarySearchWindow {
+	GtkWidget *window;
+
+	GtkWidget *bool_optmenu;
+
+	GtkWidget *from_entry;
+	GtkWidget *to_entry;
+	GtkWidget *subject_entry;
+	GtkWidget *body_entry;
+
+	GtkWidget *case_checkbtn;
+
+	GtkWidget *clear_btn;
+	GtkWidget *close_btn;
+	GtkWidget *all_btn;
+	GtkWidget *prev_btn;
+	GtkWidget *next_btn;
+
+	SummaryView *summaryview;
+} search_window;
+
+static void summary_search_create	(void);
+
+static void summary_search_execute	(gboolean	 backward,
+					 gboolean	 search_all);
+
+static void summary_search_clear	(GtkButton	*button,
+					 gpointer	 data);
+static void summary_search_prev_clicked	(GtkButton	*button,
+					 gpointer	 data);
+static void summary_search_next_clicked	(GtkButton	*button,
+					 gpointer	 data);
+static void summary_search_all_clicked	(GtkButton	*button,
+					 gpointer	 data);
+
+static void from_activated		(void);
+static void to_activated		(void);
+static void subject_activated		(void);
+static void body_activated		(void);
+
+static gboolean key_pressed		(GtkWidget	*widget,
+					 GdkEventKey	*event,
+					 gpointer	 data);
 
-static void summary_search_create(SummaryView *summaryview);
-static void summary_search_execute(GtkButton *button, gpointer data);
-static void summary_search_clear(GtkButton *button, gpointer data);
-static void from_activated(void);
-static void to_activated(void);
-static void subject_activated(void);
-static void body_activated(void);
-static void all_clicked(GtkButton *button);
-static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);
 
 void summary_search(SummaryView *summaryview)
 {
-	if (!window)
-		summary_search_create(summaryview);
+	if (!search_window.window)
+		summary_search_create();
 	else
-		gtk_widget_hide(window);
+		gtk_widget_hide(search_window.window);
 
-	gtk_widget_grab_focus(search_btn);
-	gtk_widget_grab_focus(subject_entry);
-	gtk_widget_show(window);
+	search_window.summaryview = summaryview;
+
+	gtk_widget_grab_focus(search_window.next_btn);
+	gtk_widget_grab_focus(search_window.subject_entry);
+	gtk_widget_show(search_window.window);
 }
 
-static void summary_search_create(SummaryView *summaryview)
+static void summary_search_create(void)
 {
+	GtkWidget *window;
 	GtkWidget *vbox1;
 	GtkWidget *bool_hbox;
+	GtkWidget *bool_optmenu;
 	GtkWidget *bool_menu;
 	GtkWidget *menuitem;
+	GtkWidget *clear_btn;
+
 	GtkWidget *table1;
 	GtkWidget *from_label;
+	GtkWidget *from_entry;
 	GtkWidget *to_label;
+	GtkWidget *to_entry;
 	GtkWidget *subject_label;
+	GtkWidget *subject_entry;
 	GtkWidget *body_label;
-	GtkWidget *checkbtn_hbox;
-	GtkWidget *confirm_area;
+	GtkWidget *body_entry;
 
-	window = gtk_window_new (GTK_WINDOW_DIALOG);
-	gtk_window_set_title (GTK_WINDOW (window), _("Search messages"));
-	gtk_widget_set_usize (window, 450, -1);
+	GtkWidget *checkbtn_hbox;
+	GtkWidget *case_checkbtn;
+
+	GtkWidget *confirm_area;
+	GtkWidget *close_btn;
+	GtkWidget *all_btn;
+	GtkWidget *prev_btn;
+	GtkWidget *next_btn;
+
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_window_set_title(GTK_WINDOW (window), _("Search messages"));
+	gtk_widget_set_size_request(window, 450, -1);
 	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
-	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(gtk_widget_hide_on_delete), NULL);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(key_pressed), NULL);
+	gtk_container_set_border_width(GTK_CONTAINER (window), 8);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(gtk_widget_hide_on_delete), NULL);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(key_pressed), NULL);
 	MANAGE_WINDOW_SIGNALS_CONNECT(window);
 
 	vbox1 = gtk_vbox_new (FALSE, 0);
@@ -130,6 +172,10 @@ static void summary_search_create(SummaryView *summaryview)
 	MENUITEM_ADD(bool_menu, menuitem, _("Match all of the following"), 1);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(bool_optmenu), bool_menu);
 
+	clear_btn = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
+	gtk_widget_show(clear_btn);
+	gtk_box_pack_end(GTK_BOX(bool_hbox), clear_btn, FALSE, FALSE, 0);
+
 	table1 = gtk_table_new (4, 3, FALSE);
 	gtk_widget_show (table1);
 	gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);
@@ -141,29 +187,29 @@ static void summary_search_create(SummaryView *summaryview)
 	gtk_widget_show (from_entry);
 	gtk_table_attach (GTK_TABLE (table1), from_entry, 1, 3, 0, 1,
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(from_entry), "activate",
-			   GTK_SIGNAL_FUNC(from_activated), summaryview);
+	g_signal_connect(G_OBJECT(from_entry), "activate",
+			 G_CALLBACK(from_activated), NULL);
 
 	to_entry = gtk_entry_new ();
 	gtk_widget_show (to_entry);
 	gtk_table_attach (GTK_TABLE (table1), to_entry, 1, 3, 1, 2,
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(to_entry), "activate",
-			   GTK_SIGNAL_FUNC(to_activated), summaryview);
+	g_signal_connect(G_OBJECT(to_entry), "activate",
+			 G_CALLBACK(to_activated), NULL);
 
 	subject_entry = gtk_entry_new ();
 	gtk_widget_show (subject_entry);
 	gtk_table_attach (GTK_TABLE (table1), subject_entry, 1, 3, 2, 3,
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(subject_entry), "activate",
-			   GTK_SIGNAL_FUNC(subject_activated), summaryview);
+	g_signal_connect(G_OBJECT(subject_entry), "activate",
+			 G_CALLBACK(subject_activated), NULL);
 
 	body_entry = gtk_entry_new ();
 	gtk_widget_show (body_entry);
 	gtk_table_attach (GTK_TABLE (table1), body_entry, 1, 3, 3, 4,
 			  GTK_EXPAND|GTK_FILL, 0, 0, 0);
-	gtk_signal_connect(GTK_OBJECT(body_entry), "activate",
-			   GTK_SIGNAL_FUNC(body_activated), summaryview);
+	g_signal_connect(G_OBJECT(body_entry), "activate",
+			 G_CALLBACK(body_activated), NULL);
 
 	from_label = gtk_label_new (_("From:"));
 	gtk_widget_show (from_label);
@@ -203,112 +249,134 @@ static void summary_search_create(SummaryView *summaryview)
 	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), case_checkbtn,
 			    FALSE, FALSE, 0);
 
-	backward_checkbtn =
-		gtk_check_button_new_with_label (_("Backward search"));
-	gtk_widget_show (backward_checkbtn);
-	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), backward_checkbtn,
-			    FALSE, FALSE, 0);
-
-	all_checkbtn =
-		gtk_check_button_new_with_label (_("Select all matched"));
-	gtk_widget_show (all_checkbtn);
-	gtk_box_pack_start (GTK_BOX (checkbtn_hbox), all_checkbtn,
-			    FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(all_checkbtn), "clicked",
-			   GTK_SIGNAL_FUNC(all_clicked), summaryview);
-
-	gtkut_button_set_create(&confirm_area,
-				&search_btn, _("Search"),
-				&clear_btn,  _("Clear"),
-				&close_btn,  _("Close"));
+	confirm_area = gtk_hbutton_box_new();
 	gtk_widget_show (confirm_area);
-	gtk_box_pack_start (GTK_BOX (vbox1), confirm_area, FALSE, FALSE, 0);
-	gtk_widget_grab_default(search_btn);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(confirm_area),
+				  GTK_BUTTONBOX_END);
+	gtk_box_set_spacing(GTK_BOX(confirm_area), 6);
 
-	gtk_signal_connect(GTK_OBJECT(search_btn), "clicked",
-			   GTK_SIGNAL_FUNC(summary_search_execute),
-			   summaryview);
-	gtk_signal_connect(GTK_OBJECT(clear_btn), "clicked",
-			   GTK_SIGNAL_FUNC(summary_search_clear),
-			   summaryview);
-	gtk_signal_connect_object(GTK_OBJECT(close_btn), "clicked",
-				  GTK_SIGNAL_FUNC(gtk_widget_hide),
-				  GTK_OBJECT(window));
+	close_btn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	GTK_WIDGET_SET_FLAGS(close_btn, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(confirm_area), close_btn, TRUE, TRUE, 0);
+	gtk_widget_show(close_btn);
+
+	all_btn = gtk_button_new_from_stock(_("Find all"));
+	GTK_WIDGET_SET_FLAGS(all_btn, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(confirm_area), all_btn, TRUE, TRUE, 0);
+	gtk_widget_show(all_btn);
+
+	prev_btn = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
+	GTK_WIDGET_SET_FLAGS(prev_btn, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(confirm_area), prev_btn, TRUE, TRUE, 0);
+	gtk_widget_show(prev_btn);
+
+	next_btn = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
+	GTK_WIDGET_SET_FLAGS(next_btn, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(confirm_area), next_btn, TRUE, TRUE, 0);
+	gtk_widget_show(next_btn);
+
+	gtk_box_pack_start (GTK_BOX (vbox1), confirm_area, FALSE, FALSE, 0);
+	gtk_widget_grab_default(next_btn);
+
+	g_signal_connect(G_OBJECT(clear_btn), "clicked",
+			 G_CALLBACK(summary_search_clear), NULL);
+	g_signal_connect(G_OBJECT(all_btn), "clicked",
+			 G_CALLBACK(summary_search_all_clicked), NULL);
+	g_signal_connect(G_OBJECT(prev_btn), "clicked",
+			 G_CALLBACK(summary_search_prev_clicked), NULL);
+	g_signal_connect(G_OBJECT(next_btn), "clicked",
+			 G_CALLBACK(summary_search_next_clicked), NULL);
+	g_signal_connect_closure
+		(G_OBJECT(close_btn), "clicked",
+		 g_cclosure_new_swap(G_CALLBACK(gtk_widget_hide),
+				     window, NULL),
+		 FALSE);
+
+	search_window.window = window;
+	search_window.bool_optmenu = bool_optmenu;
+	search_window.from_entry = from_entry;
+	search_window.to_entry = to_entry;
+	search_window.subject_entry = subject_entry;
+	search_window.body_entry = body_entry;
+	search_window.case_checkbtn = case_checkbtn;
+	search_window.clear_btn = clear_btn;
+	search_window.close_btn = close_btn;
+	search_window.all_btn = all_btn;
+	search_window.prev_btn = prev_btn;
+	search_window.next_btn = next_btn;
 }
 
-static void summary_search_execute(GtkButton *button, gpointer data)
+static void summary_search_execute(gboolean backward, gboolean search_all)
 {
-	SummaryView *summaryview = data;
-	GtkCTree *ctree = GTK_CTREE(summaryview->ctree);
-	GtkCTreeNode *node;
+	SummaryView *summaryview = search_window.summaryview;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
 	MsgInfo *msginfo;
 	gboolean bool_and;
 	gboolean case_sens;
-	gboolean backward;
-	gboolean search_all;
 	gboolean all_searched = FALSE;
 	gboolean matched;
 	gboolean body_matched;
-	gchar *from_str, *to_str, *subject_str, *body_str;
+	const gchar *from_str, *to_str, *subject_str, *body_str;
 	StrFindFunc str_find_func;
+	gboolean valid;
 
 	if (summary_is_locked(summaryview)) return;
 	summary_lock(summaryview);
 
-	bool_and = GPOINTER_TO_INT
-		(menu_get_option_menu_active_user_data
-			(GTK_OPTION_MENU(bool_optmenu)));
+	model = GTK_TREE_MODEL(summaryview->store);
+
+	bool_and = menu_get_option_menu_active_index
+		(GTK_OPTION_MENU(search_window.bool_optmenu));
 	case_sens = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON(case_checkbtn));
-	backward = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON(backward_checkbtn));
-	search_all = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON(all_checkbtn));
+		(GTK_TOGGLE_BUTTON(search_window.case_checkbtn));
 
 	if (case_sens)
 		str_find_func = str_find;
 	else
 		str_find_func = str_case_find;
 
-	from_str    = gtk_entry_get_text(GTK_ENTRY(from_entry));
-	to_str      = gtk_entry_get_text(GTK_ENTRY(to_entry));
-	subject_str = gtk_entry_get_text(GTK_ENTRY(subject_entry));
-	body_str    = gtk_entry_get_text(GTK_ENTRY(body_entry));
+	from_str    = gtk_entry_get_text(GTK_ENTRY(search_window.from_entry));
+	to_str      = gtk_entry_get_text(GTK_ENTRY(search_window.to_entry));
+	subject_str = gtk_entry_get_text(GTK_ENTRY(search_window.subject_entry));
+	body_str    = gtk_entry_get_text(GTK_ENTRY(search_window.body_entry));
 
 	if (search_all) {
-		gtk_clist_freeze(GTK_CLIST(ctree));
-		gtk_clist_unselect_all(GTK_CLIST(ctree));
-		node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
+		summary_unselect_all(summaryview);
+		valid = gtk_tree_model_get_iter_first(model, &iter);
 		backward = FALSE;
 	} else if (!summaryview->selected) {
 		if (backward)
-			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list_end);
+			valid = gtkut_tree_model_get_iter_last(model, &iter);
 		else
-			node = GTK_CTREE_NODE(GTK_CLIST(ctree)->row_list);
-
-		if (!node) {
+			valid = gtk_tree_model_get_iter_first(model, &iter);
+		if (!valid) {
 			summary_unlock(summaryview);
 			return;
 		}
 	} else {
+		valid = gtkut_tree_row_reference_get_iter
+			(model, summaryview->selected, &iter);
+		if (!valid) {
+			summary_unlock(summaryview);
+			return;
+		}
+
 		if (backward)
-			node = gtkut_ctree_node_prev
-				(ctree, summaryview->selected);
+			valid = gtkut_tree_model_prev(model, &iter);
 		else
-			node = gtkut_ctree_node_next
-				(ctree, summaryview->selected);
+			valid = gtkut_tree_model_next(model, &iter);
 	}
 
 	if (*body_str)
 		main_window_cursor_wait(summaryview->mainwin);
 
 	for (;;) {
-		if (!node) {
+		if (!valid) {
 			gchar *str;
 			AlertValue val;
 
 			if (search_all) {
-				gtk_clist_thaw(GTK_CLIST(ctree));
 				break;
 			}
 
@@ -326,24 +394,22 @@ static void summary_search_execute(GtkButton *button, gpointer data)
 				str = _("End of list reached; continue from beginning?");
 
 			val = alertpanel(_("Search finished"), str,
-					 _("Yes"), _("No"), NULL);
+					 GTK_STOCK_YES, GTK_STOCK_NO, NULL);
 			if (G_ALERTDEFAULT == val) {
 				if (backward)
-					node = GTK_CTREE_NODE
-						(GTK_CLIST(ctree)->row_list_end);
+					valid  = gtkut_tree_model_get_iter_last
+						(model, &iter);
 				else
-					node = GTK_CTREE_NODE
-						(GTK_CLIST(ctree)->row_list);
-
+					valid = gtk_tree_model_get_iter_first
+						(model, &iter);
 				all_searched = TRUE;
-
-				manage_window_focus_in(window, NULL, NULL);
+				manage_window_focus_in(search_window.window,
+						       NULL, NULL);
 			} else
 				break;
 		}
 
-
-		msginfo = gtk_ctree_node_get_row_data(ctree, node);
+		gtk_tree_model_get(model, &iter, S_COL_MSG_INFO, &msginfo, -1);
 		body_matched = FALSE;
 
 		if (bool_and) {
@@ -397,14 +463,16 @@ static void summary_search_execute(GtkButton *button, gpointer data)
 		}
 
 		if (matched) {
-			if (search_all)
-				gtk_ctree_select(ctree, node);
-			else {
+			if (search_all) {
+				gtk_tree_selection_select_iter
+					(summaryview->selection, &iter);
+			} else {
 				if (messageview_is_visible
 					(summaryview->messageview)) {
 					summary_unlock(summaryview);
-					summary_select_node
-						(summaryview, node, TRUE, TRUE);
+					summary_select_row
+						(summaryview, &iter,
+						 TRUE, TRUE);
 					summary_lock(summaryview);
 					if (body_matched) {
 						messageview_search_string
@@ -412,15 +480,18 @@ static void summary_search_execute(GtkButton *button, gpointer data)
 							 body_str, case_sens);
 					}
 				} else {
-					summary_select_node
-						(summaryview, node, FALSE, TRUE);
+					summary_select_row
+						(summaryview, &iter,
+						 FALSE, TRUE);
 				}
 				break;
 			}
 		}
 
-		node = backward ? gtkut_ctree_node_prev(ctree, node)
-				: gtkut_ctree_node_next(ctree, node);
+		if (backward)
+			valid = gtkut_tree_model_prev(model, &iter);
+		else
+			valid = gtkut_tree_model_next(model, &iter);
 	}
 
 	if (*body_str)
@@ -431,42 +502,55 @@ static void summary_search_execute(GtkButton *button, gpointer data)
 
 static void summary_search_clear(GtkButton *button, gpointer data)
 {
-	gtk_editable_delete_text(GTK_EDITABLE(from_entry),    0, -1);
-	gtk_editable_delete_text(GTK_EDITABLE(to_entry),      0, -1);
-	gtk_editable_delete_text(GTK_EDITABLE(subject_entry), 0, -1);
-	gtk_editable_delete_text(GTK_EDITABLE(body_entry),    0, -1);
+	gtk_editable_delete_text(GTK_EDITABLE(search_window.from_entry),
+				 0, -1);
+	gtk_editable_delete_text(GTK_EDITABLE(search_window.to_entry),
+				 0, -1);
+	gtk_editable_delete_text(GTK_EDITABLE(search_window.subject_entry),
+				 0, -1);
+	gtk_editable_delete_text(GTK_EDITABLE(search_window.body_entry),
+				 0, -1);
+}
+
+static void summary_search_prev_clicked(GtkButton *button, gpointer data)
+{
+	summary_search_execute(TRUE, FALSE);
+}
+
+static void summary_search_next_clicked(GtkButton *button, gpointer data)
+{
+	summary_search_execute(FALSE, FALSE);
+}
+
+static void summary_search_all_clicked(GtkButton *button, gpointer data)
+{
+	summary_search_execute(FALSE, TRUE);
 }
 
 static void from_activated(void)
 {
-	gtk_widget_grab_focus(to_entry);
+	gtk_widget_grab_focus(search_window.to_entry);
 }
 
 static void to_activated(void)
 {
-	gtk_widget_grab_focus(subject_entry);
+	gtk_widget_grab_focus(search_window.subject_entry);
 }
 
 static void subject_activated(void)
 {
-	gtk_button_clicked(GTK_BUTTON(search_btn));
+	gtk_button_clicked(GTK_BUTTON(search_window.next_btn));
 }
 
 static void body_activated(void)
 {
-	gtk_button_clicked(GTK_BUTTON(search_btn));
+	gtk_button_clicked(GTK_BUTTON(search_window.next_btn));
 }
 
-static void all_clicked(GtkButton *button)
-{
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
-		gtk_widget_set_sensitive(backward_checkbtn, FALSE);
-	else
-		gtk_widget_set_sensitive(backward_checkbtn, TRUE);
-}
-
-static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
+static gboolean key_pressed(GtkWidget *widget, GdkEventKey *event,
+			    gpointer data)
 {
 	if (event && event->keyval == GDK_Escape)
-		gtk_widget_hide(window);
+		gtk_widget_hide(search_window.window);
+	return FALSE;
 }
